package net.universe_factory.minecraft.test.generic;

import java.util.List;
import java.util.Random;

import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLeaves;
import net.minecraft.block.BlockLog;
import net.minecraft.block.BlockSapling;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import net.minecraft.world.gen.feature.WorldGenBigTree;
import net.minecraft.world.gen.feature.WorldGenTrees;
import net.minecraft.world.gen.feature.WorldGenerator;
import net.minecraftforge.event.terraingen.TerrainGen;
import net.universe_factory.minecraft.test.Test;

public class GenericWood {
	public final Info info;

	public final Block planks;
	public final Block log;
	public final BlockLeaves leaves;
	public final BlockSapling sapling;

	public GenericWood(Info info) {
		this.info = info;

		planks = new Planks();
		log = new Log();
		leaves = new Leaves();
		sapling = new Sapling();
	}

	public void register() {
		GameRegistry.registerBlock(planks, "blockWood" + info.getName());
		GameRegistry.registerBlock(log, "blockLog" + info.getName());
		GameRegistry.registerBlock(leaves, "blockLeaves" + info.getName());
		GameRegistry.registerBlock(sapling, "blockSapling" + info.getName());

		GameRegistry.addRecipe(new ItemStack(planks, 4, 0), "#", '#', log);
	}

	private class Planks extends Block {
		public Planks() {
			super(Material.wood);

			setHardness(2.0f);
			setResistance(5.0f);
			setStepSound(Block.soundTypeWood);
			setBlockName("wood" + info.getName());
			setCreativeTab(CreativeTabs.tabBlock);
			setHarvestLevel("axe", 0);

			setBlockTextureName(Test.MODID + ":planks_" + info.getName().toLowerCase());
		}
	}

	private class Log extends BlockLog {
		private IIcon sideIcon;
		private IIcon topIcon;

		public Log() {
			setBlockName("log" + info.getName());
			setHarvestLevel("axe", 0);

			setBlockTextureName(Test.MODID + ":log");
		}

		@Override
		@SideOnly(Side.CLIENT)
		public void registerBlockIcons(IIconRegister iconRegister) {
			sideIcon = iconRegister.registerIcon(this.getTextureName() + "_" + info.getName().toLowerCase());
			topIcon = iconRegister.registerIcon(this.getTextureName() + "_" + info.getName().toLowerCase() + "_top");
		}

		@Override
		@SideOnly(Side.CLIENT)
		protected IIcon getSideIcon(int meta) {
			return sideIcon;
		}

		@Override
		@SideOnly(Side.CLIENT)
		protected IIcon getTopIcon(int meta) {
			return topIcon;
		}
	}

	private class Leaves extends BlockLeaves {
		private String[] nameArray = new String[] { info.getName() };
		private IIcon icon;
		private IIcon iconOpaque;

		public Leaves() {
			setBlockName("leaves" + info.getName());

			setBlockTextureName(Test.MODID + ":leaves");

			setGraphicsLevel(true);
		}

		@Override
		@SideOnly(Side.CLIENT)
		public void registerBlockIcons(IIconRegister iconRegister) {
			icon = iconRegister.registerIcon(this.getTextureName() + "_" + info.getName().toLowerCase());
			iconOpaque = iconRegister.registerIcon(this.getTextureName() + "_" + info.getName().toLowerCase() + "_opaque");
		}

		@Override
		@SideOnly(Side.CLIENT)
		public IIcon getIcon(int side, int meta) {
			return (field_150127_b == 0) ? icon : iconOpaque;
		}

		@Override
		public String[] func_150125_e() {
			return nameArray;
		}

		@Override
		public int quantityDropped(Random random) {
			return random.nextInt(20) == 0 ? 1 : 0;
		}

		@Override
		public Item getItemDropped(int p_149650_1_, Random p_149650_2_, int p_149650_3_) {
			return Item.getItemFromBlock(sapling);
		}
	}

	private class Sapling extends BlockSapling {
		public Sapling() {
			setBlockName("sapling" + info.getName());

			setHardness(0.0f);
			setStepSound(soundTypeGrass);

			setBlockTextureName(Test.MODID + ":sapling_" + info.getName().toLowerCase());
		}

		@Override
		@SideOnly(Side.CLIENT)
		public void registerBlockIcons(IIconRegister iconRegister) {
			blockIcon = iconRegister.registerIcon(this.getTextureName());
		}

		@Override
		@SideOnly(Side.CLIENT)
		public IIcon getIcon(int side, int meta) {
			return blockIcon;
		}

		@Override
		@SideOnly(Side.CLIENT)
		public void getSubBlocks(Item item, CreativeTabs tabs, List list) {
			list.add(new ItemStack(item, 1, 0));
		}

		/** Grows the tree */
		@Override
		public void func_149878_d(World world, int x, int y, int z, Random random) {
			if (!TerrainGen.saplingGrowTree(world, random, x, y, z))
				return;
			
			WorldGenerator generator = new WorldGenGenericTrees(true, GenericWood.this);
			
			world.setBlock(x, y, z, Blocks.air, 0, 4);
			
			if (!generator.generate(world, random, x, y, z))
		                world.setBlock(x, y, z, this, 0, 4);
		}
	}

	public static abstract class Info {
		public abstract String getName();
		public abstract int getMinTreeHeight();
	}
}
